/*
 * deviced
 *
 * Copyright (c) 2015 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef __EXTCON_H__
#define __EXTCON_H__

#include "core/common.h"
#include <hw/external_connection.h>

/**
 * Extcon cable name is shared with kernel extcon class.
 * So do not change below strings.
 */
#define EXTCON_CABLE_USB              EXTERNAL_CONNECTION_USB
#define EXTCON_CABLE_USB_HOST         EXTERNAL_CONNECTION_USB_HOST
#define EXTCON_CABLE_TA               EXTERNAL_CONNECTION_TA
#define EXTCON_CABLE_HDMI             EXTERNAL_CONNECTION_HDMI
#define EXTCON_CABLE_DOCK             EXTERNAL_CONNECTION_DOCK
#define EXTCON_CABLE_MIC_IN           EXTERNAL_CONNECTION_MIC
#define EXTCON_CABLE_HEADPHONE_OUT    EXTERNAL_CONNECTION_HEADPHONE

struct extcon_ops {
	const char *name;
	int status;
	int enabled;
	void (*init)(void *data);
	void (*exit)(void *data);
	int (*update)(int status);
};

/* Status for External Connectors */
typedef enum {
	USB_DISCONNECTED,
	USB_CONNECTED
} extcon_usb_state_e;

typedef enum {
	HDMI_DISCONNECTED,
	HDMI_CONNECTED
} extcon_hdmi_state_e;

typedef enum {
	USBHOST_DISCONNECTED,
	USBHOST_CONNECTED
} extcon_usbhost_state_e;

typedef enum {
	EARJACK_DICONNECTED	= 0,
	EARJACK_3WIRE		= 0x1,
	EARJACK_4WIRE		= 0x3,
	EARJACK_TVOUT		= 0x10
} extcon_earjack_state_e;

#define EXTCON_OPS_REGISTER(dev) \
static void __CONSTRUCTOR__ extcon_init(void) \
{ \
	/**
	 * If there is no predefined status value,
	 * default status will set as a negative value(-1).
	 * It will be updated as the initial value in booting time.
	 */ \
	if (!dev.status) \
		dev.status = -1; \
	add_extcon(&dev); \
} \
static void __DESTRUCTOR__ extcon_exit(void) \
{ \
	remove_extcon(&dev); \
}

#define CONNECTED(val) ((val) != 0)

void add_extcon(struct extcon_ops *dev);
void remove_extcon(struct extcon_ops *dev);

int extcon_get_status(const char *name);
int extcon_enable_device(const char *name);
int extcon_disable_device(const char *name);
#endif /* __EXTCON_H__ */
